package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import java.util.logging.Level;

import org.apache.commons.lang3.StringUtils;
import org.junit.Test;

/**
 * Test cases for the basic expression language.
 * 
 * @author Frank Wibbelink
 */
public class ExpressionTest {

	/**
	 * Correct basic expressions
	 */
	@Test
	public void correctExpressionParsing() {
		AllTests.parseFile("simpleExpression.boppi");
		assertThat(AllTests.log, empty());
	}

	/**
	 * Basic expressions with syntax errors
	 */
	@Test
	public void wrongExpressionParsing() {
		AllTests.parseString("");
		assertThat(AllTests.log, not(empty()));

		AllTests.parseString("~");
		assertThat(AllTests.log, not(empty()));

		AllTests.parseString("0A");
		assertThat(AllTests.log, hasSize(1));

		AllTests.parseString("do");
		assertThat(AllTests.log, hasSize(1));

		AllTests.parseString("true true");
		assertThat(AllTests.log, hasSize(1));
	}

	/**
	 * Correct basic expressions
	 */
	@Test
	public void correctExpressionChecking() {
		AllTests.checkFile("simpleExpression.boppi");
		assertThat(AllTests.log, is(empty()));
	}

	/**
	 * Basic expressions with type errors
	 */
	@Test
	public void wrongExpressionChecking() {
		AllTests.checkString("+true");
		assertThat(AllTests.log, hasSize(1));

		AllTests.checkString("5 || true");
		assertThat(AllTests.log, hasSize(1));

		AllTests.checkString("6 + 'c'");
		assertThat(AllTests.log, hasSize(1));

		AllTests.checkString("4 + print(5, 6)");
		assertThat(AllTests.log, hasSize(1));

		AllTests.checkString("print(print(3, 5))");
		assertThat(AllTests.log, hasSize(1));
	}

	/**
	 * Correct basic expressions
	 */
	@Test
	public void correctExpressionGeneration() {
		AllTests.compileAndRunFile("simpleExpression.boppi");
		assertThat(AllTests.log, is(empty()));

		int n = 12;
		String expression = StringUtils.repeat("1+(", n) + "1" + StringUtils.repeat(")", n);

		AllTests.withWarnings(() -> {
			AllTests.compileAndRunString("print(" + expression + ")", "" + n);
			assertThat(AllTests.log, hasSize(1));
			assertThat(AllTests.log.get(0).getLevel(), is(Level.WARNING));
		});
	}

	/**
	 * Basic expression with run-time error
	 */
	@Test
	public void wrongExpressionGeneration() {
		AllTests.compileAndRunString("1/0");
		assertThat(AllTests.log, hasSize(1));
		assertThat(AllTests.log.get(0).getMessage(), containsString("zero"));
	}

}
